/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.handlers.basic;

import org.w3c.dom.Node;

import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.event.failed.FailedEvent;
import com.ibm.rfidic.handlers.IHandler;
import com.ibm.rfidic.handlers.InterningException;
import com.ibm.rfidic.mdm.IElement;
import com.ibm.rfidic.mdm.MasterDataManager;
import com.ibm.rfidic.metadata.IAttributeMetaData;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.ValueFactory;

/**
 * This is a Handler for SGLN readpoints and readlocations.  It can handle the following formats
 * 		<readPoint>
 * 			<id>urn:epc:id:sgln:1234567.123.1</id>
 *      </readPoint>
 * 
 * This assumes that the external ID for SGLN urn:epc:id:sgln:1234567.123.1 will be 
 * 1234567.123.1 in the masterdata.
 * 
 * * example setup:
 * 	1) metadata : RFIDIC_HOME/etc/EPCISDocumentMetaData.xml
 * 		 <tns:AttributeMetaData>
                <Name>readPoint</Name>
                <Path>readPoint/id</Path>
                <Key>false</Key>
                <Internable>true</Internable>
                <Required>true</Required>
                <Handler>com.ibm.rfidic.handlers.basic.SimpleGLNHandler</Handler>
                <Vocabularies>
                  <Vocabulary>urn:epcglobal:epcis:vtype:ReadPoint</Vocabulary>
                </Vocabularies>
                <TableName>Event</TableName>
                <ColumnName>readpoint</ColumnName>
                <Type>string</Type>
                <Unique>false</Unique>
                <MaxSize>80</MaxSize>
            </tns:AttributeMetaData>
     
     2) MasterData
        <!-- vocabularies need to match-->
        <Vocabulary type="urn:rfidic:vocabularytype:location">
           <VocabularyElementList>
           		<VocabularyElement id="1234567.123.1">
           			<attribute id="urn:rfidic:location:name">Ground Floor</attribute>
           			<attribute id="urn:rfidic:location:address">Las Vegas, Nevada</attribute>
                </VocabularyElement>
           </VocabularyElementList>
           
     Note that the handler supports both "readPoint/id" and NOT "readPoint" as a path. In the first case
     the intern( IAttributeMetaData, IAttributeValue) method is called. In the the second case the
     intern( IAttributeMetaData, Node) method. The second case the unintern method must produce the full XML fragment
     , which isn't shown here. 

 *
 */
public class SimpleGLNHandler implements IHandler
{
	public static final String copyright = "(c) Copyright IBM Corporation 2006.";
	
    public static final String PREFIX = "urn:epc:id:sgln:";

    private MasterDataManager m_mdm;

    public SimpleGLNHandler() throws Exception
    {
        m_mdm = MasterDataManager.getInstance();
        if (m_mdm == null)
            throw new RFIDICException("Unable to initinalize the MDM API.");
    }

    
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#unIntern(int)
     */
    public IAttributeValue unIntern(int internID) throws InterningException
    {
        // get the Location object for the internal ID
        IElement location = m_mdm.getLocation(internID);
        
        // reconstruct the original value
        String s = location.getName();
        String s2 = PREFIX + s.toString();
        return ValueFactory.getValue(s2);
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#intern(com.ibm.rfidic.metadata.IAttributeMetaData, org.w3c.dom.Node)
     */
    public IAttributeValue intern(IAttributeMetaData attributeMD, Node node)
            throws InterningException
    {
        throw new InterningException("SimpleGLNSHandler cannot be used for uninterning of full XML fragments. ");
    }
    
 
 
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#intern(com.ibm.rfidic.metadata.IAttributeMetaData, com.ibm.rfidic.value.IAttributeValue)
     */
    public IAttributeValue intern(IAttributeMetaData attributeMD,
            IAttributeValue value) throws InterningException
    {
        String[] vocabs = attributeMD.getVocabularies();
        if ((vocabs == null) || (vocabs.length == 0)) {
            String error="The handler searched for the vocabularies that were specified in the metadata. "+
            "However, the vocabularies specified for the '"+attributeMD.getName()+"' attribute could not be found in the master data. "+
            "Import the vocabularies for the '"+attributeMD.getName()+"' attribute.";             

            throw new InterningException(error);
        }
        String vocabType = vocabs[0];
        
        // derive externalID
        String externalID = stripPrefix (value.toString());
        
        // do MDM lookup
        IElement location = m_mdm.getLocation(externalID);
        if (location == null) {
            String error="The handler did not find a location with an SGLN value of '"+value+"' for the '"+attributeMD.getName()+"' attribute. "+
            "The external ID is '"+externalID+"'. Import the location.";
            
            throw new InterningException(error, attributeMD.getName(), value, vocabType,
                    FailedEvent.RCMissingLocation);
        }
        
        // check if the location is actually in the vocabulary that was specified in the 
        // metadata. Otherwise the event views won't work.
        String locationVocabulary=location.getVocabulary().getName();
        int matchedVocab=-1;
        for (int i=0;i<vocabs.length;i++) {
            if (locationVocabulary.equals(vocabs[i]))
                matchedVocab=i;
        }        
        if (matchedVocab==-1) {
            String error="Location with external ID '"+externalID+"' was found in vocabulary '"+locationVocabulary+
            "', but the metadata does not include this for attribute '"+attributeMD.getName()+
            "'. Update the metadata.";
            throw new InterningException(error, attributeMD.getName(), value, vocabType,
                    FailedEvent.RCMissingLocation);
        }
        
        // return internal ID
        int intern = location.getId();
        return ValueFactory.getValue(intern);
    }

    /**
     *  Helper method to strip uri prefixes from a SGLN and returns keys
     *  input : urn:epc:id:sgln:1234567.123.1 ---> 1234567.123.1
     * 
     * @param s URI to be stripped of prefixes
     * @return text without prefixes
     */
    protected String stripPrefix(String s)
    {
        if (s == null)
            return null;
        StringBuffer buf = new StringBuffer ();
        int index = 0;
        for (index = 0; index < s.length(); index ++)
        {
            char c = s.charAt(index);
            if (!Character.isDigit(c))
                continue;
            else 
                break;
            // end of prefix, return the rest
        }
        return s.substring(index);
    }
}
