/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.handlers.basic;

import org.w3c.dom.Node;

import com.ibm.rfidic.event.failed.FailedEvent;
import com.ibm.rfidic.handlers.IHandler;
import com.ibm.rfidic.handlers.InterningException;
import com.ibm.rfidic.mdm.IElement;
import com.ibm.rfidic.mdm.IVocabulary;
import com.ibm.rfidic.mdm.MasterDataManager;
import com.ibm.rfidic.metadata.IAttributeMetaData;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.ValueFactory;

/**
 * This class will handle general vocabularies.
 * 
 * example setup:
 * 	1) metadata : RFIDIC_HOME/etc/EPCISDocumentMetaData.xml
 * 		<tns:AttributeMetaData>
                <Name>bizStep</Name>
                <!-- this xpath is relative to the parent -->
                <Path>bizStep</Path>
                <Key>false</Key>
                <Internable>true</Internable>
                <Handler>com.ibm.rfidic.handlers.basic.SimpleVocabHandler</Handler>
                <Vocabularies>
                  <Vocabulary>urn:epcglobal:epcis:vtype:BusinessStep</Vocabulary>
                </Vocabularies>
                <Required>true</Required>
                <TableName>Event</TableName>
                <ColumnName>bizstep</ColumnName>
                <Type>string</Type>
                <Unique>false</Unique>
                <MaxSize>80</MaxSize>
            </tns:AttributeMetaData>
      
     2) MasterData
        <!-- vocabularies need to match-->
     	<Vocabulary type="urn:epcglobal:epcis:vtype:BusinessStep">
		<VocabularyElementList>
			<VocabularyElement id="Shipped">
				<attribute id="description">description of shipped bizStep</attribute>
			</VocabularyElement>
		</VocabularyElementList>
	  </Vocabulary>
 * 
 *
 */
public class SimpleVocabHandler implements IHandler
{
	public static final String copyright = "(c) Copyright IBM Corporation 2006.";
    protected MasterDataManager m_mdm;

    public SimpleVocabHandler() throws Exception
    {
        m_mdm = MasterDataManager.getInstance();
        if (m_mdm == null)
            throw new InterningException("Unable to initialize MDM API.");
    }

    
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#intern(com.ibm.rfidic.metadata.IAttributeMetaData, com.ibm.rfidic.value.IAttributeValue)
     */
    public IAttributeValue intern(IAttributeMetaData attributeMD,
            IAttributeValue value) throws InterningException
    {
        String[] vocabs = attributeMD.getVocabularies();
        String name = attributeMD.getName();
        
        if (vocabs==null || vocabs.length == 0) 
        {
            String error="The handler searched all the vocabularies that were specified in the metadata. However, there were no vocabularies specified for the  '"+name+
            "'  attribute. Specify a vocabulary for the '"+name+"' attribute in the metadata.";
            throw new InterningException(error, name, value,
                    null,
                    FailedEvent.RCMissingVocabularyElement);
        }
        
        String vocabType=null;
        
        // iterate over the vocabularies that are specified in the metadata and search for the
        // attribute value 
        for (int i = 0; (vocabs != null) && (i < vocabs.length); i++)
        {
            vocabType = vocabs[i];
            
            IVocabulary vocab = m_mdm.getVocabulary(vocabType);
            if (vocab == null)
            {
                String error="The handler searched for the vocabularies that were specified in the metadata. "+
                  "However, the vocabularies specified for the '"+name+"' attribute could not be found in the master data. "+
                  "Import the vocabularies for the "+name+" attribute.";
                
                throw new InterningException(error, name, value, vocabType,
                        FailedEvent.RCMissingVocabularyElement);
            }
            
            IElement elem = vocab.getElement(value.toString());
            if (elem == null)
                continue;
            
            // return the internal ID of that vocabulary element
            int intern = elem.getId();
            return ValueFactory.getValue(intern);
        }
        
        String error = "The handler searched all of the vocabularies that were specified in the metadata. "+
        	"However, a vocabulary element with the value '"+value+"' was not found in the '"+name+"' attribute. "+
        	"Import the vocabulary element.";
         
        throw new InterningException(error, name, value,
                vocabType,
                FailedEvent.RCMissingVocabularyElement);
    }

    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#intern(com.ibm.rfidic.metadata.IAttributeMetaData, org.w3c.dom.Node)
     */    
    public IAttributeValue intern(IAttributeMetaData attributeMD, Node node)
            throws InterningException
    {
        
        throw new InterningException("Handler does not handle XML fragments.");
    }

    
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#unIntern(int)
     */   
    public IAttributeValue unIntern(int internID) throws InterningException
    {
        IVocabulary vocab = m_mdm.getVocabularyByElementId(internID);
        if (vocab == null) {
            throw new InterningException(
                    "Getting vocabulary failed for internal id '" + internID+"'.");
        }
        IElement  elem=vocab.getElement(internID);
        if (elem == null) {
            throw new InterningException(
                    "Getting vocabulary element failed for id '" + internID + 
                    "' in vocabulary '"+vocab.getName()+"'.");
        }
        String external = elem.getName();
        return ValueFactory.getValue(external);
    }
    
}
