/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.delivery;

import java.util.Iterator;
import java.util.List;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.message.builder.TextMessageBuilder;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.TypeFactory;

public class JMSDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String    NAME = "JMS delivery method";
  private static final String    DESCRIPTION = "Writes notifications into a JMS queue.";

  public static final IParameter    JNDI_QCF_PARAMETER;
  public static final IParameter    JNDI_Q_PARAMETER;
  public static final IParameter    MESSAGE_PARAMETER;

  private static final IParameter[] parameters;

  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    final IAttributeType stringType = typeFactory.getStringType(255, 0);

    JNDI_QCF_PARAMETER = new Parameter("JNDI name of queue connection factory", stringType, true, null);
    JNDI_Q_PARAMETER = new Parameter("JNDI name of JMS queue", stringType, true, null);
    MESSAGE_PARAMETER = new Parameter("Message", stringType, true, null);

    /*
     * Example JNDI name for JMS connection factory:
     * "jms/alert_cf"
     * Example JNDI name for JMS queue:
     * "jms/alert_q"
     */
    parameters = new IParameter[] { JNDI_QCF_PARAMETER, JNDI_Q_PARAMETER, MESSAGE_PARAMETER };
  }

  private TextMessageBuilder        messageBuilder;

  public JMSDeliveryMethod()
  {
    super(NAME, DESCRIPTION);
    messageBuilder = new TextMessageBuilder();
  }

  public IParameter[] getParameters()
  {
    return parameters;
  }

  public void processNotifications(List notifications, IDeliveryPoint deliveryPoint)
  {
    if ( notifications == null || notifications.size() == 0 )
      return;

    for ( Iterator ix = notifications.iterator(); ix.hasNext(); )
    {
      INotification notification = (INotification) ix.next();

      try
      {
        final IParameterValue[] parameterValues = notification.getDeliveryValues(deliveryPoint);

        final String connectionFactoryName = (String) Utility.findValue(parameterValues, JNDI_QCF_PARAMETER).getSQLValue();
        final String queueName = (String) Utility.findValue(parameterValues, JNDI_Q_PARAMETER).getSQLValue();
        final String message = (String) Utility.findValue(parameterValues, MESSAGE_PARAMETER).getSQLValue();

        final InitialContext initialContext = new InitialContext();
        final ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup(connectionFactoryName);
        final Queue queue = (Queue) initialContext.lookup(queueName);

        if ( connectionFactory == null || queue == null )
        {
          /*
           * FIXME: Replace this by an error code.
           */
          System.err.println("The queue connection factory name, " + connectionFactoryName + 
              ", or the queue name, " + queueName + ", are not properly defined.");
        }
        else
        {
          String text = messageBuilder.buildMessage(notification, message);

          final Connection connection = connectionFactory.createConnection();
          connection.start();
          final Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
          final TextMessage outMessage = session.createTextMessage();
          outMessage.setText(text);

          /*
           * Put the message into the queue.
           */
          session.createProducer(queue).send(outMessage);

          session.close();
          connection.close();
        }
      }
      catch ( NamingException e )
      {
        e.printStackTrace();
      }
      catch ( JMSException e )
      {
        e.printStackTrace();
      }
      catch(RFIDICException e)
      {
        e.printStackTrace();
      }
      catch ( RuntimeException e )
      {
        e.printStackTrace();
      }
    }
  }

  public String[] getOutputChannels()
  {
    return null;
  }
}
