/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.message.builder;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;

import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.value.TypeFactory;

/**
 * @author vcrisan
 */

public class TextMessageBuilder
{
    private IParameter[] outputParameters;

    public TextMessageBuilder()
    {
        final TypeFactory typeFactory = TypeFactory.getInstance();
        outputParameters = new IParameter[1];
        outputParameters[0] = new Parameter("$timestamp", typeFactory.getDateType(), true, null);
    }

    public IParameter[] getOutputParameters(ISubscription subscription)
    {
        return outputParameters;
    }

    public String buildMessage(INotification notification, String messageTemplate)
    {
        final ISubscription subscription = notification.getSubscription();

        String word = findParameter(messageTemplate);
        while (word != null)
        {
            /*
             * (result == null) means that no replacement was found.
             */
            String result = replaceSystemParameters(messageTemplate, word);

            if (result == null)
            {
                result = replaceSubscriptionParameters(messageTemplate, word, subscription);
            }

            if (result == null)
            {
                result = replaceNotificationParameters(messageTemplate, word, notification);
            }

            if (result == null)
            {
                result = messageTemplate.replaceAll("\\" + word, "<unknown>");
            }
            messageTemplate = result;
            word = findParameter(messageTemplate);
        }
        return messageTemplate;
    }

    /**
     * These parameters are global to the Alert system, and not specific to the
     * notification.
     */
    String replaceSystemParameters(String template, String word)
    {
        if (word.equals("$timestamp"))
        {
            final Date date = (new GregorianCalendar()).getTime();
            final String timeStampText = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS Z").format(date);
            template = template.replaceAll("\\" + word, timeStampText);
            return template;
        }
        else
        {
            return null;
        }
    }

    /**
     * These parameters are global to the Alert system, and not specific to the
     * notification.
     */
    String replaceSubscriptionParameters(String template, String word, final ISubscription subscription)
    {
        if (word.equals("$alert_name"))
        {
            template = template.replaceAll("\\" + word, subscription.getAlert().getClass().getName());
            return template;
        }
        else if (word.equals("$subscriber"))
        {
            template = template.replaceAll("\\" + word, subscription.getSubscriber());
            return template;
        }
        else
        {
            return null;
        }
    }

    /**
     * These parameters are provided by the notification.
     */
    String replaceNotificationParameters(String template, String word, final INotification notification)
    {
        final IParameterValue[] parameterValues = notification.getAlertValues();
        if (parameterValues != null)
        {
            for (int i = 0; i < parameterValues.length; i++)
            {
                final String name = parameterValues[i].getParameter().getName();
                if (word.equals("$" + name))
                {
                    final String value = parameterValues[i].getValue().getSQLValue().toString();
                    template = template.replaceAll("\\" + word, value);
                    return template;
                }
            }
        }
        return null;
    }

    private String findParameter(String text)
    {
        final int length = text.length();
        final int start = text.indexOf("$");
        if (start < 0)
        {
            return null;
        }

        // FIXME: skip over double-$ sign

        int end = start + 1;
        while (end < length)
        {
            final char c = text.charAt(end);
            if (c != '_' && !Character.isLetterOrDigit(c))
            {
                break;
            }
            end++;
        }

        if (end > start + 1)
        {
            return text.substring(start, end);
        }

        return null;
    }
}
