/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.nativealerts;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import com.ibm.rfidic.alerts.AlertsAdapter;
import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.innate.INativeAlert;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.metadata.IAttributeMetaData;
import com.ibm.rfidic.metadata.IEntityMetaData;
import com.ibm.rfidic.path.IAttributePath;
import com.ibm.rfidic.path.PathDictionary;
import com.ibm.rfidic.value.IAttributeValue;

/** 
 * This class is a super abstract class for all Native Alerts. It contains common
 * methods that are needed by all Native Alerts and do not need to be altered by
 * the Solution Developer.
 */
public abstract class SuperNativeAlert extends Entity implements INativeAlert
{
  /** Constructor */
  public SuperNativeAlert(String name, String description)
  {
    super(name, description);
  }

  /** This method is called after the constructor. 
   * There is no need to alter this method for Native Alerts. */
  public boolean initialize()
  {
    return true;
  }
  
  /**
   * This method is called when a new event arrives in the RFIDIC system. The
   * method is responsible for handling all subscriptions that have been made
   * to this alert. It returns a list of notifications to be passed to the
   * Delivery Method(s). This method may be altered for performance tuning 
   * if necessary. This method calls the abstract method evaluateCondition
   * which is overridden by a solution developer implementing a specific 
   * native alert.
   */
  public List evaluateConditions(IEvent event) 
  {
    final Collection subscriptions = AlertsAdapter.getInstance()
        .getSubscriptionRegistry().getAllForAlert(getId());
    final ArrayList notifications = new ArrayList();
    for (Iterator iterator = subscriptions.iterator(); iterator.hasNext();) 
    {
      final ISubscription subscription = (ISubscription) iterator.next();
      if(subscription.isEnabled())
      {
        List returnNotifications = evaluateCondition(event, subscription);
        if (returnNotifications != null)
          notifications.addAll(returnNotifications);
      }
    }
    return notifications;
  }
  
  /** This abstract method must be provided by the solution developer
   * who implements a native alert in the alert class that extends this 
   * class. It returns a List of INotification objects which result from 
   * the processing of the alert based on the IEvent and ISubscription 
   * passed in or null may be returned if the alert condition is false.
   */
  abstract protected List evaluateCondition(IEvent event, ISubscription subscription);

  /** This method handles subscriptions. 
   * There is no need to alter this method for Native Alerts. */
  public void addSubscription(ISubscription subscription)
  {
  }

  /** This method handles subscriptions. 
   * There is no need to alter this method for Native Alerts. */
  public void removeSubscription(ISubscription subscription)
  {
  }
  
  /** This method registers this Alert with the Native engine. 
   * There is no need to alter this method for Native Alerts. */
  public void register()
  {
    AlertsAdapter.getInstance().getNativeEngine().registerAlert(this);
  }

  /** This method unregisters this Alert with the Native engine. 
   * There is no need to alter this method for Native Alerts. */
  public void unregister()
  {
    AlertsAdapter.getInstance().getNativeEngine().unregisterAlert(this);
  }
  
  /** 
   * This is a standard method that will return an attribute value from an
   * event given the attribute's name. If the attribute being retrieved is a 
   * standard EPCIS attribute then the name should a value from the 
   * EventConstants object. Otherwise it should be the name of the 
   * extended attribute defined in the event metadata for the solution.  
   */
  public IAttributeValue getAttributeValue(IEvent event, String attrName) 
  {
    return(getAttributeValue(event, attrName, ""));
  }

  /**
   * This method is an extension to the overloaded method getAttributeValue.  This method
   * allows the passing of a third parameter which is a String suffix to the attribute name.
   * This is used to retrieve attribute values for epcLists.  Since lists are suffixed
   * with #1, #2, etc, this String can be passed in to retrieve the individual elements in the list.
   */
  public IAttributeValue getAttributeValue(IEvent event, String attrName, String PathSuffix) 
  {
    IEntityMetaData eventMD = event.getEntityMetaData();
    IAttributeMetaData ia = eventMD.getAttributeMetaDataByName(attrName);
    String attrPath = eventMD.getAttributeAbsolutePath(ia)+PathSuffix;

    IAttributePath attributePath = PathDictionary.getInstance().getAttributePath(attrPath);
    IAttributeValue attributeValue = event.getAttributeValue(attributePath);
    return attributeValue;
  }
}
